#include "entity.h"

#include "const.h"
#include "utils.h"
#include "player.h"

void createEntity(Entity* entity, sf::Vector2f pos, float r, sf::Color color, int value) {
	entity->moveInfo.pos = pos;
	for (int i = 0; i<NB_SHAPES_ENTITY; i++)
		entity->shapes[i].setPosition(pos);

	entity->radius = r;
	entity->shapes[0].setRadius(r);
	entity->shapes[0].setFillColor(color);
	entity->shapes[0].setOutlineColor(sf::Color(80, 80, 80, 128));
	entity->shapes[0].setOutlineThickness(1.3);

	entity->shapes[1].setRadius(r - 2);
	entity->shapes[1].setFillColor(sf::Color(60, 60, 60, 128));
	entity->shapes[1].setOutlineColor(sf::Color::Transparent);
	entity->shapes[1].setOutlineThickness(1);



	for (int i = 0; i<NB_SHAPES_ENTITY; i++) {
		float hRadius = entity->shapes[i].getRadius();
		entity->shapes[i].setOrigin(sf::Vector2f(hRadius, hRadius));
	}

	entity->value = value;

	entity->active = true;
}

bool updateEntity(Entity* entity, float dt, std::vector<Line>& segments) {
           
      float dampening = 0.989989f;
      float accSpeed = 1023.0123456789f;
      
      sf::Vector2f lastPos = entity->moveInfo.pos;
      
      bool moved = false;

      entity->moveInfo.vel += entity->moveInfo.acc * dt;


      entity->moveInfo.pos += entity->moveInfo.vel * dt;

      // if(!moved)
      entity->moveInfo.vel *= dampening;
      
      entity->moveInfo.acc *= 0.0f;


      bool killed = intersectEntityWithLevel(entity, segments, lastPos, dt);
      
      if(entity->moveInfo.pos.x<0)
            entity->moveInfo.pos.x = WINDOW_WIDTH; 
      else if(entity->moveInfo.pos.x>WINDOW_WIDTH)
            entity->moveInfo.pos.x = 0;
      
      if(entity->moveInfo.pos.y<0)
            entity->moveInfo.pos.y = WINDOW_HEIGHT; 
      if(entity->moveInfo.pos.y>WINDOW_HEIGHT)
            entity->moveInfo.pos.y = 0;
      
      for(int i=0 ; i<NB_SHAPES_ENTITY ; i++)
            entity->shapes[i].setPosition(entity->moveInfo.pos);

      return killed;
}

bool intersectEntityWithLevel(Entity* entity, std::vector<Line>& segments, sf::Vector2f lastPos, float dt) {
      int nbSegments = segments.size();
      
      sf::Vector2f pos = entity->moveInfo.pos;

      sf::Vector2f inter, interMin(INFINITY, INFINITY);
      float tInter, tMin = INFINITY;
      bool intersected = false;
      Line lineInter;
      for (int i = 0; i < nbSegments; i++) {
            Line line = segments[i];
            if (getEdgeIntersection(lastPos, pos, line.a, line.b, &inter, &tInter) && tInter < tMin) {
                  interMin = inter;
                  tMin = tInter;
                  lineInter = line;
                  intersected |= true;
            }
      }

      if (intersected) {
		  if (lineInter.type == LineType::BAD) {
			  entity->active = false;
			  entity->lastPlayerTouched->score += entity->value;
			  return true;
		  }
		  sf::Vector2f dir = lastPos - inter;
		  float lgth = length(dir);
		  dir /= lgth;
		  float tNear = tMin*0.9f;
		  entity->moveInfo.pos = lastPos + dir*tNear;


		  // float dx = lineInter.b.x - lineInter.a.x;
		  // float dy = lineInter.b.y - lineInter.a.y;
		  // sf::Vector2f normal = sf::Vector2f(-dy, dx);
		  // float dotNV = dot(entity->moveInfo.vel, normal);
		  // sf::Vector2f reflected = entity->moveInfo.vel - 2.0f*dotNV*normal;
		  // entity->moveInfo.vel = reflected;

		  // sf::Vector2f line = normalize(lineInter.b - lineInter.a);
		  // // sf::Vector2f factor(dot(line, entity->moveInfo.vel), dot(line, normal));
		  // float dotLD = dot(dir, line);
		  // sf::Vector2f factor;
		  // if(dotLD>0.)
		  //       factor = sf::Vector2f(0.0f, 1.0f);
		  // else factor = sf::Vector2f(1.0f, 0.0f);
		  // // entity->moveInfo.vel.x *= factor.x;
		  // // entity->moveInfo.vel.y *= factor.y;


		  float dx = lineInter.b.x - lineInter.a.x;
		  float dy = lineInter.b.y - lineInter.a.y;
		  float delta = dy / dx;
		  if (std::abs(delta)>0.5f)
			  entity->moveInfo.vel.x *= -0.45f;
		  else entity->moveInfo.vel.y *= -0.45f;

		  entity->moveInfo.pos += entity->moveInfo.vel*dt;

	  }
    return false;
}
