#include "player.h"

#include <cmath>


#include <iostream>
#include <algorithm>

#include "utils.h"


void createPlayer(Player* player, int id, sf::Vector2f pos, float r, sf::Color color) {
	player->id = id;

	player->moveInfo.pos = pos;
	player->shape.setPosition(pos);
	player->radius = r;
	player->shape.setRadius(r);
	player->shape.setFillColor(color);
	//      player->shape.setOutlineColor(sf::Color(255, 255, 255));
	//      player->shape.setOutlineThickness(8);

	player->outerShapes[0].setOutlineThickness(1);
	player->outerShapes[0].setOutlineColor(sf::Color(200, 200, 200));
	player->outerShapes[0].setFillColor(sf::Color::Transparent);
	player->outerShapes[0].setPosition(pos);
	player->outerShapes[0].setRadius(r + 1);

	player->outerShapes[1].setOutlineThickness(1);
	player->outerShapes[1].setOutlineColor(sf::Color(189, 189, 189));
	player->outerShapes[1].setFillColor(sf::Color::Transparent);
	player->outerShapes[1].setPosition(pos);
	player->outerShapes[1].setRadius(r + 2);

	player->animShape.setOutlineThickness(1);
	player->animShape.setOutlineColor(sf::Color(255, 18, 99, 133));
	player->animShape.setFillColor(sf::Color(128, 128, 128, 68));
	player->animShape.setPosition(pos);
	player->animShape.setRadius(0);


	float hRadius = player->radius;
	player->shape.setOrigin(sf::Vector2f(hRadius, hRadius));

	hRadius = player->animShape.getRadius();
	player->animShape.setOrigin(sf::Vector2f(hRadius, hRadius));

	for (int i = 0; i<NB_SHAPES; i++) {
		hRadius = player->outerShapes[i].getRadius();
		player->outerShapes[i].setOrigin(sf::Vector2f(hRadius, hRadius));
	}
}

void movePlayer(Player* player, float dt, sf::Vector2f& lastPos, bool joystick) {
           
      float dampening = 0.9389f;
      float accSpeed = 1321.0123456789f;

      float maxVel = 823.123456789f;
      sf::Vector2f acc(0.0f, 0.0f);

      bool moved = false;

      lastPos = player->moveInfo.pos;
      if(!joystick) {
            float degueu = 1;
      
            if(player->goUp) {
                  if(player->moveInfo.vel.y>0)
                        player->moveInfo.vel.y *= degueu;
                  acc.y -= accSpeed;
                  moved = true;
            }
            if(player->goDown) {
                  if(player->moveInfo.vel.y<0)
                        player->moveInfo.vel.y *= degueu;
                  acc.y += accSpeed;
                  moved = true;
            }
            if(player->goRight) {
                  if(player->moveInfo.vel.x<0)
                        player->moveInfo.vel.x *= degueu;
                  acc.x += accSpeed;
                  moved = true;
            }
            if(player->goLeft) {
                  if(player->moveInfo.vel.x>0)
                        player->moveInfo.vel.x *= degueu;
                  acc.x -= accSpeed;
                  moved = true;
            }
            player->moveInfo.vel += acc * dt;
      }
      else
            player->moveInfo.vel += player->moveInfo.acc * dt;

      if(player->moveInfo.vel.x < -maxVel)
            player->moveInfo.vel.x = -maxVel;
      else if(player->moveInfo.vel.x > maxVel)
            player->moveInfo.vel.x = maxVel;

      if(player->moveInfo.vel.y < -maxVel)
            player->moveInfo.vel.y = -maxVel;
      else if(player->moveInfo.vel.y > maxVel)
            player->moveInfo.vel.y = maxVel;

      player->moveInfo.acc *= 0.0f;
      
      player->moveInfo.pos += player->moveInfo.vel * dt;

      if(!moved)
            player->moveInfo.vel *= dampening;
      
      
      player->shape.setPosition(player->moveInfo.pos);
      for(int i=0 ; i<NB_SHAPES ; i++)
            player->outerShapes[i].setPosition(player->moveInfo.pos);
      player->animShape.setPosition(player->moveInfo.pos);

}

void updatePlayer(Player* player, float dt, std::vector<Line>& segments, std::vector<Entity>& entities, bool joystick) {
      sf::Vector2f lastPos;
      movePlayer(player, dt, lastPos, joystick);
      intersectPlayerWithLevel(player, segments, lastPos, dt);
      if (player->animImpulse)
            updatePlayerImpulse(player, entities, segments);
}

void intersectPlayerWithLevel(Player* player, std::vector<Line>& segments, sf::Vector2f lastPos, float dt) {
      int nbSegments = segments.size();
	
      sf::Vector2f pos = player->moveInfo.pos;

      sf::Vector2f inter, interMin(INFINITY, INFINITY);
      float tInter, tMin = INFINITY;
      bool intersected = false;
      Line lineInter;
      for (int i = 0; i < nbSegments; i++) {
            Line line = segments[i];
            if (getEdgeIntersection(lastPos, pos, line.a, line.b, &inter, &tInter) && tInter < tMin) {
                  interMin = inter;
                  tMin = tInter;
                  lineInter = line;
                  intersected |= true;
            }
      }

      if (intersected) {
            sf::Vector2f dir = lastPos - inter;
            float length = std::sqrt(dir.x*dir.x + dir.y*dir.y);
            dir /= length;
            float tNear = tMin*0.9f;
            player->moveInfo.pos = lastPos + dir*tNear;

            
            // float dx = lineInter.b.x - lineInter.a.x;
            // float dy = lineInter.b.y - lineInter.a.y;
            // sf::Vector2f normal = sf::Vector2f(-dy, dx);
            // float dotNV = dot(player->moveInfo.vel, normal);
            // sf::Vector2f reflected = player->moveInfo.vel - 2.0f*dotNV*normal;
            // player->moveInfo.vel = reflected;

            // sf::Vector2f line = normalize(lineInter.b - lineInter.b);
            // sf::Vector2f factor(dot(line, player->moveInfo.vel), dot(line, normal));
            // player->moveInfo.vel.x *= factor.x;
            // player->moveInfo.vel.y *= factor.y;

            //player->moveInfo.vel *= -0.45f;
            float dx = lineInter.b.x - lineInter.a.x;
            float dy = lineInter.b.y - lineInter.a.y;
            float delta = dy / dx;
            if(std::abs(delta)>0.5f)
                  player->moveInfo.vel.x *= -0.45f;
            else player->moveInfo.vel.y *= -0.45f;

            

            player->moveInfo.pos += player->moveInfo.vel*dt;
      }

}

void applyForce(Player* player, Entity* entity) {
		entity->lastPlayerTouched = player;
      sf::Vector2f dir = entity->moveInfo.pos - player->moveInfo.pos;
      float length = std::sqrt(dir.x*dir.x+dir.y*dir.y);
      dir /= length;
      entity->moveInfo.acc += dir * (3123.123456789f+std::sin(1.0f - player->keyTime)*100.0123456789f);
}

void displayPlayer(sf::RenderWindow& window, Player* player, float elapsedTime) {

      if (player->animImpulse) {

            float elapsedTimeAnim = elapsedTime - player->timeBeginAnimImpulse;

            player->keyTime = elapsedTimeAnim / player->timerAnimImpulse;
            float radius = player->radius + IMPULSE_RADIUS * player->keyTime;
            player->animShape.setRadius(radius);
            float alpha = std::max(1.0f-player->keyTime, 0.001f)*160;
            sf::Color c0 = player->animShape.getFillColor();
            player->animShape.setFillColor(sf::Color(c0.r, c0.g, c0.b, alpha));
            player->animShape.setOrigin(sf::Vector2f(radius, radius));
            sf::Color c1 = player->animShape.getOutlineColor();
            player->animShape.setOutlineColor(sf::Color(c1.r, c1.g, c1.b, 0));
            window.draw(player->animShape);

            
            if (elapsedTimeAnim > player->timerAnimImpulse) {
                  player->timeBeginAnimImpulse = 0.0f;
                  player->animImpulse = false;
            }
            
      }

      window.draw(player->shape);
      for (int i = 0; i<NB_SHAPES; i++)
            window.draw(player->outerShapes[i]);



}

void updatePlayerImpulse(Player* player, std::vector<Entity>& entities, std::vector<Line>& segments) {
      int nbEntities = entities.size();
      for (int i = 0; i<nbEntities; i++) {
		  if (!entities[i].active)
			  continue;

            float dist = distance(entities[i].moveInfo.pos, player->moveInfo.pos) - entities[i].radius;
            if (dist <= player->keyTime*IMPULSE_RADIUS &&
                entityPlayerVisibility(player, &entities[i], segments, &dist)) {
                  applyForce(player, &entities[i]);
            }
      }

}
