#include <cmath>

#include <iostream>

#include <vector>

#include <SFML/Audio.hpp>
#include <SFML/System.hpp>
#include <SFML/Window.hpp>
#include <SFML/Graphics.hpp>

#include "const.h"
#include "entity.h"
#include "player.h"
#include "level.h"
#include "utils.h"
#include "scene.h"

enum GameState {
      TitleScreen,
      Game,
      EndScreen
};

GameState currentGameState = GameState::TitleScreen;

Level level;

sf::Vector2f newLineStart, newLineEnd;
LineType newLineType;


Player player;
Player player2;

std::vector<Entity> entities;

std::vector<Scene> scenes;
int currentScene;
SceneState currentState;

sf::Font font;

sf::Clock clk;

sf::SoundBuffer soundBuffer;
sf::SoundBuffer soundBufferDeactivate;
sf::SoundBuffer soundBufferSelect;
sf::Sound sound;


bool running = true;


float elapsedTime, elapsedFrameTime;

bool hasJoystick0 = false;
bool hasJoystick1 = false;

using ::std::cout;
using ::std::cerr;
using ::std::endl;

bool EDIT_MODE = false;

void playSFXAnimate() {
	sound.setBuffer(soundBufferDeactivate);
	sound.play();
}

void playSFXSelect() {
	sound.setBuffer(soundBufferSelect);
	sound.play();
}

void initLevelFromScene(Level* lvl, Scene* sc) {
      loadLevel(lvl, sc->levelFilepath, &entities, &player.moveInfo.pos, &player2.moveInfo.pos);
      player.moveInfo.vel = sf::Vector2f(0.0f, 0.0f);
      player2.moveInfo.vel = sf::Vector2f(0.0f, 0.0f);
      level.width = WINDOW_WIDTH;
      level.height = WINDOW_HEIGHT;
}

void nextScene() {
      currentScene++;
      initLevelFromScene(&level, &scenes[currentScene]);
}


void handleEvent(sf::RenderWindow& window, sf::Event& event) {
	while (window.pollEvent(event)) {
		if (event.type == sf::Event::JoystickButtonPressed) {
			if (event.joystickButton.button == 7 && currentGameState == GameState::TitleScreen) {

				initLevelFromScene(&level, &scenes[currentScene]);

				currentGameState = GameState::Game;
				currentState = SceneState::intro;
				clk.restart();

				playSFXSelect();
			}
			else if (event.joystickButton.button == 7 && currentGameState == GameState::EndScreen) {
				player.score = 0;
				player.totalScore = 0;
				player2.totalScore = 0;

				currentScene = 0;
				initLevelFromScene(&level, &scenes[currentScene]);

				currentGameState = GameState::TitleScreen;
				currentState = SceneState::intro;
				clk.restart();
				playSFXSelect();
			}
			else if (event.joystickButton.button == 7 && currentState == SceneState::intro) {
				player.score = 0;
                player.frameAnimImpulse = 0;
				player2.score = 0;
                player2.frameAnimImpulse = 0;
				player.animImpulse = false;
				player2.animImpulse = false;
    player.keyTime = 0.0f;
    player2.keyTime = 0.0f;
                  player.timeBeginAnimImpulse = 0.0f;
                  player2.timeBeginAnimImpulse = 0.0f;


				initLevelFromScene(&level, &scenes[currentScene]);

				currentState = SceneState::game;
				clk.restart();
				playSFXSelect();
			}
			else if (event.joystickButton.button == 7 && currentState == SceneState::score) {
				if (currentScene < scenes.size() - 1) {
					currentState = SceneState::intro;
					currentScene++;
				}
				else if (currentScene == scenes.size() - 1) {
					// on passe  l'cran de fin
					player.totalScore += player.score;
					player2.totalScore += player2.score;

					currentGameState = GameState::EndScreen;
				}
				playSFXSelect();
				clk.restart();
			}
		}
		if (event.type == sf::Event::MouseButtonPressed) {
			if (EDIT_MODE) {
				newLineStart = sf::Vector2f(event.mouseButton.x, event.mouseButton.y);
				if (event.mouseButton.button == sf::Mouse::Left)
					newLineType = LineType::NORMAL;
				else if (event.mouseButton.button == sf::Mouse::Right)
					newLineType = LineType::BAD;
			}
			else {
				sf::Vector2i mousePos = sf::Mouse::getPosition(window);
				Entity e;
				createEntity(&e, sf::Vector2f(mousePos.x, mousePos.y), 8, sf::Color(200, 200, 200, 200));
				entities.push_back(e);
			}
		}
		if (event.type == sf::Event::MouseButtonReleased) {
			if (EDIT_MODE) {
				newLineEnd = sf::Vector2f(event.mouseButton.x, event.mouseButton.y);
				addSegmentToLevel(&level, newLineStart, newLineEnd, newLineType);
			}
		}
		if (event.type == sf::Event::Closed) {
			// end the program
			running = false;
		}
		if (event.type == sf::Event::KeyPressed) {
			switch (event.key.code) {
			case sf::Keyboard::Escape:
				running = false;
				break;
			case sf::Keyboard::Q:
				nextScene();
				break;
			case P1_UP:
				player.goUp = true;
				break;
			case P1_DOWN:
				player.goDown = true;
				break;
			case P1_LEFT:
				player.goLeft = true;
				break;
			case P1_RIGHT:
				player.goRight = true;
				break;
			case P2_UP:
				player2.goUp = true;
				break;
			case P2_DOWN:
				player2.goDown = true;
				break;
			case P2_LEFT:
				player2.goLeft = true;
				break;
			case P2_RIGHT:
				player2.goRight = true;
				break;
			case sf::Keyboard::E:
				EDIT_MODE = !EDIT_MODE;
				break;
			case sf::Keyboard::S:
				saveLevel(&level, "monlevel.txt", &entities, &player.moveInfo.pos, &player2.moveInfo.pos);
				break;
			case sf::Keyboard::P:
				level.segments.clear();
				entities.clear();
				loadLevel(&level, "monlevel.txt", &entities, &player.moveInfo.pos, &player2.moveInfo.pos);
				break;
			case sf::Keyboard::BackSpace:
				level.segments.pop_back();
				break;

			case P1_IMPULSE:
				if (currentState == SceneState::game && launchPlayerAnimImpulse(&player, elapsedTime)) {
					sound.setBuffer(soundBuffer);
					sound.play();
					// for (int i = 0; i<entities.size(); i++) {
					//       float dist = distance(entities[i].moveInfo.pos, player.moveInfo.pos);
					//       if (dist < IMPULSE_RADIUS &&
					//           entityPlayerVisibility(&player, &entities[i], level.segments)) {
					//             applyForce(&player, &entities[i]);
					//       }
					// }
				}

				break;

			case P2_IMPULSE:
				if (currentState == SceneState::game && launchPlayerAnimImpulse(&player2, elapsedTime)) {
					sound.setBuffer(soundBuffer);
					sound.play();
					// for (int i = 0; i<entities.size(); i++) {
					//       float dist = distance(entities[i].moveInfo.pos, player2.moveInfo.pos);
					//       if (dist < IMPULSE_RADIUS &&
					//           entityPlayerVisibility(&player2, &entities[i], &level)) {
					//             applyForce(&player2, &entities[i]);
					//       }
					// }
					// // cout << entity.moveInfo.acc.x << endl;
				}

				break;
			default:
				break;
			}

		}


		else if (event.type == sf::Event::KeyReleased) {
			switch (event.key.code) {
			case P1_UP:
				player.goUp = false;
				break;
			case P1_DOWN:
				player.goDown = false;
				break;
			case P1_LEFT:
				player.goLeft = false;
				break;
			case P1_RIGHT:
				player.goRight = false;
				break;

			case P2_UP:
				player2.goUp = false;
				break;
			case P2_DOWN:
				player2.goDown = false;
				break;
			case P2_LEFT:
				player2.goLeft = false;
				break;
			case P2_RIGHT:
				player2.goRight = false;
				break;
			}

		}
	}
}

int main() {
      
      srand(time(NULL));
      sf::ContextSettings settings;
      settings.antialiasingLevel = MSAA_LEVEL;

      // create the window
      sf::RenderWindow window(sf::VideoMode(WINDOW_WIDTH, WINDOW_HEIGHT), "Proto", sf::Style::Default, settings);
      window.setVerticalSyncEnabled(true);
      window.setKeyRepeatEnabled(false);
      sf::Event event;

      if (!font.loadFromFile("assets/fonts/os.ttf")) {
                  cerr << "Can't load font";
      }
      
      hasJoystick0 = sf::Joystick::isConnected(0);
      if(hasJoystick0)
            cout << "Player 1's Joystick is Connected" << endl;
      hasJoystick1= sf::Joystick::isConnected(1);
      if(hasJoystick1)
            cout << "Player 2's Joystick is Connected" << endl;

      // init scenes
      Scene intro, tuto, sc2;

      initScene(&tuto, 1, "assets/levels/tuto.txt", 30, "Day 1");
      initScene(&intro, 2, "assets/levels/level0.txt", 60, "Day 2");
      initScene(&sc2, 3, "assets/levels/test0.txt", 60, "Day 3");

      scenes.push_back(tuto);
      scenes.push_back(intro);
      scenes.push_back(sc2);

      currentScene = 0;

      createPlayer(&player, 1, sf::Vector2f(WINDOW_WIDTH*0.25f, WINDOW_HEIGHT*0.5f), 10, COLOR_PLAYER1);
      createPlayer(&player2, 2, sf::Vector2f(WINDOW_WIDTH*0.75f, WINDOW_HEIGHT*0.5f), 10, COLOR_PLAYER2);

      // int n = 256;
      // entities.resize(n*2);

      // float theta = 0.0f;
      // float r = 128;
      // float step = 2.0f * 3.1415926535f / n;
      // for (int i = 0; i<n; i++) {
      //       float x = -600+i*3;
      //       float y = (r*std::sin(theta*5.32)) + 16.3f*std::cos(theta*19.12f);
      //       theta += step;
      //       createEntity(&entities[i], sf::Vector2f(230+WINDOW_WIDTH*0.5f + x, WINDOW_HEIGHT*0.5f + y), 8, sf::Color(200, 200, 200, 200));
      // }
      // for (int i = n; i<2*n; i++) {
      //       float x = r*std::cos(theta);
      //       float y = r*std::sin(theta);
      //       theta += step;
      //       createEntity(&entities[i], sf::Vector2f(-230+WINDOW_WIDTH*0.5f + x, WINDOW_HEIGHT*0.5f + y), 8, sf::Color(200, 200, 200, 200));
      // }
            
	  if (!soundBuffer.loadFromFile("assets/sfx/impulse.wav")) 
		  cerr << "Unable to load impulse sfx sound\n";
	  if (!soundBufferDeactivate.loadFromFile("assets/sfx/deactivate.wav")) 
		  cerr << "Unable to load deactivation sfx sound\n";
	  

	  if (!soundBufferSelect.loadFromFile("assets/sfx/select.wav")) 
		  cerr << "Unable to load select sfx sound\n";
	  
      

      while (running) {
            static float previous_seconds = clk.getElapsedTime().asSeconds();
            //cout << previous_seconds << endl;
            elapsedTime = clk.getElapsedTime().asSeconds();
            elapsedFrameTime = elapsedTime - previous_seconds;
            previous_seconds = elapsedTime;

            handleEvent(window, event);

            if (currentGameState == GameState::TitleScreen) {
                  window.clear(COLOR_BACKGROUND);

                  sf::Text text;
                  // select the font
                  text.setFont(font); // font is a sf::Font
                  // set the string to display
                  text.setString("Shepherds simulator 2016\n");
                  // set the character size
                  text.setCharacterSize(64); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, WINDOW_WIDTH / 3);
                  // set the color
                  text.setColor(COLOR_PLAYER2);

                  sf::FloatRect textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, WINDOW_HEIGHT / 3.0f));
                  window.draw(text);

                  text.setString("Plug two controllers then press start");
                  // set the character size
                  text.setCharacterSize(34); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, 2 * WINDOW_WIDTH / 3);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + 2 * textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, 2 * WINDOW_HEIGHT / 3.0f));
                  window.draw(text);

                   text.setString(" \n(Guess the Controls)\n");
                  // set the character size
                  text.setCharacterSize(16); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, 3 * WINDOW_WIDTH / 4);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + 2 * textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, 3 * WINDOW_HEIGHT / 4.0f));
                 
                  window.draw(text);
            }
            else if (currentGameState == GameState::EndScreen) {
                  window.clear(COLOR_BACKGROUND);

                  sf::Text text;
                  // select the font
                  text.setFont(font); // font is a sf::Font
                  // set the string to display
                  text.setString("Shepherds simulator 2016");
                  // set the character size
                  text.setCharacterSize(64); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, WINDOW_WIDTH / 3);
                  // set the color
                  text.setColor(COLOR_PLAYER2);

                  sf::FloatRect textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, WINDOW_HEIGHT / 3.0f));
                  window.draw(text);

                  text.setString("A game by Jules and Hubert\n Press start to restart or Esc to quit");
                  // set the character size
                  text.setCharacterSize(34); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, 2 * WINDOW_WIDTH / 3);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + 2 * textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, 2 * WINDOW_HEIGHT / 3.0f));
                  window.draw(text);

                  text.setString("Le gras, c'est la vie.\n  _ Karadoc.");
                  // set the character size
                  text.setCharacterSize(16); // in pixels, not points!
                  text.setPosition(2*WINDOW_WIDTH / 3, 3 * WINDOW_WIDTH / 4);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + 2 * textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(2*WINDOW_WIDTH / 3.0f, 3 * WINDOW_HEIGHT / 4.0f));
                  window.draw(text);

                  window.draw(text);
            }
            else if (currentState == SceneState::intro) {
                  window.clear(COLOR_BACKGROUND);

                  sf::Text text;
                  // select the font
                  text.setFont(font); // font is a sf::Font
                  // set the string to display
                  text.setString(scenes[currentScene].textCinematique + "\nPress start to ... start ");
                  // set the character size
                  text.setCharacterSize(44); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 4+64, WINDOW_WIDTH / 4);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  window.draw(text);
            }
            else if (currentState == SceneState::score) {
                  window.clear(COLOR_BACKGROUND);

                  sf::Text text;
                  // select the font
                  text.setFont(font); // font is a sf::Font
                  // set the string to display
                  text.setString("Scores");
                  if (currentScene == scenes.size() - 1)
                        text.setString("Final scores");

                  // set the character size
                  text.setCharacterSize(44); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, 100);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  window.draw(text);

                  sf::Text textScoreP1;
                  // select the font
                  textScoreP1.setFont(font); // font is a sf::Font
                  // set the string to display
                  textScoreP1.setString("Shepherd 1: " + std::to_string(player.totalScore));
                  // set the character size
                  textScoreP1.setCharacterSize(34); // in pixels, not points!
                  textScoreP1.setPosition(WINDOW_WIDTH / 2, 300);
                  // set the color
                  textScoreP1.setColor(COLOR_PLAYER1);
                  window.draw(textScoreP1);

                  sf::Text textScoreP2;
                  // select the font
                  textScoreP2.setFont(font); // font is a sf::Font
                  // set the string to display
                  textScoreP2.setString("Shepherd 2: " + std::to_string(player2.totalScore));
                  // set the character size
                  textScoreP2.setCharacterSize(34); // in pixels, not points!
                  textScoreP2.setPosition(WINDOW_WIDTH / 2, 375);
                  // set the color
                  textScoreP2.setColor(COLOR_PLAYER2);
                  window.draw(textScoreP2);

                  text.setString("Press start to continue");
                  // set the character size
                  text.setCharacterSize(34); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 2, 2 * WINDOW_WIDTH / 3);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  sf::FloatRect textRect = text.getLocalBounds();
                  text.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + 2 * textRect.height / 3.0f);
                  text.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, 2 * WINDOW_HEIGHT / 3.0f));
                  window.draw(text);
            }
            else if (currentState == SceneState::game) {
                  if(hasJoystick0) {
                        float sensibilityThreshold = 16.0f;
                        float yAxis = sf::Joystick::getAxisPosition(0, sf::Joystick::Y);
                        if(std::abs(yAxis)<sensibilityThreshold)
                              yAxis = 0.0f;
                        float xAxis = sf::Joystick::getAxisPosition(0, sf::Joystick::X);
                        if(std::abs(xAxis)<sensibilityThreshold)
                              xAxis = 0.0f;
                        player.moveInfo.acc += sf::Vector2f(xAxis, yAxis) * 19.0123456789f;
                        if(sf::Joystick::isButtonPressed(0, 0)) {
                              if (launchPlayerAnimImpulse(&player, elapsedTime)) {
                                    sound.setBuffer(soundBuffer);
                                    sound.play();
                              }
                        }
                  }
                  if(hasJoystick1) {
                        float sensibilityThreshold = 16.0f;
                        float yAxis = sf::Joystick::getAxisPosition(1, sf::Joystick::Y);
                        if(std::abs(yAxis)<sensibilityThreshold)
                              yAxis = 0.0f;
                        float xAxis = sf::Joystick::getAxisPosition(1, sf::Joystick::X);
                        if(std::abs(xAxis)<sensibilityThreshold)
                              xAxis = 0.0f;
                        player2.moveInfo.acc += sf::Vector2f(xAxis, yAxis) * 19.0123456789f;
                        if (sf::Joystick::isButtonPressed(1, 0)) {
                              if (launchPlayerAnimImpulse(&player2, elapsedTime)) {
                                    sound.setBuffer(soundBuffer);
                                    sound.play();
                              }
                        }
                  }

                  updatePlayer(&player, elapsedFrameTime, level.segments, entities, hasJoystick0);
                  updatePlayer(&player2, elapsedFrameTime, level.segments, entities, hasJoystick1);

                  bool killed = false;

                  for (int i = 0; i<entities.size(); i++) {
                        if (entities[i].active) {
                              killed |= updateEntity(&entities[i], elapsedFrameTime, level.segments);
                        }
                  }
                  if (killed)
                        playSFXAnimate();
            
                  window.clear(COLOR_BACKGROUND);
            
                  drawLevel(&level, window);

                  const int nbEntities = entities.size();

                  int nbActiveEntities = 0;
                  for (int i = 0; i<nbEntities; i++) {
                  
                        if (!entities[i].active)
                              continue;

                        nbActiveEntities++;
                        
                        updateEntity(&entities[i], elapsedFrameTime, level.segments);                  

                        for(int j=0 ; j<nbEntities ; j++) {
                              if(i==j)
                                    continue;
                              Entity& e0 = entities[i];
                              Entity& e1 = entities[j];
                              sf::Vector2f dir = e1.moveInfo.pos - e0.moveInfo.pos;
                              float dist = std::max(length(dir), 0.00001f);
                              dir = normalize(dir);
                              if(dist<(e0.radius+e1.radius)*1.33f) {
                                    // float energy = std::max(length(e0.moveInfo.vel + e1.moveInfo.vel), 0.001f);
                                    // e0.moveInfo.vel -= -dir * 0.043123456789f * energy;
                                    // e1.moveInfo.vel -=  dir * 0.043123456789f * energy;
                                    // dir = normalize(dir);
                                    // e0.moveInfo.pos -= dir*(e0.radius+0.01f);
                                    // e1.moveInfo.pos += dir*(e1.radius+0.01f);

                                    // updateEntity(&entities[i], elapsedFrameTime, level.segments);
                                    // updateEntity(&entities[j], elapsedFrameTime, level.segments);

                              

                                    //e0.moveInfo.pos += e0.moveInfo.vel * elapsedFrameTime;
                                    //e1.moveInfo.pos += e1.moveInfo.vel * elapsedFrameTime;
                                    //e0.moveInfo.acc = e0.moveInfo.vel * 12.12f;
                                    //e1.moveInfo.acc = e1.moveInfo.vel * 12.12f;

                                    float force = 13.123456789f * (189.123456789f * 16 * 16 ) / (dist * dist) ;
                                    dir *= force;
                                    applyForceEntity(e1, dir);
                        
                              }
                             
                        
                        }
                  
                  
                        sf::Color c = sf::Color(70, 70, 70, 100); 
                        float dist;
                        if (entityPlayerVisibility(&player, &entities[i], level.segments, &dist)) {


                              if(dist < IMPULSE_RADIUS) {
                                    c = sf::Color(128, 128, 20, 128); 
                                    sf::Vertex l[] = {
                                          sf::Vertex(entities[i].moveInfo.pos, c),
                                          sf::Vertex(player.moveInfo.pos, c)
                                    };
                                    window.draw(l, 2, sf::Lines);
                              }
                              else if(dist < IMPULSE_RADIUS*2.5f) {
                                    c = sf::Color(128, 128, 128, 64); 
                                    sf::Vertex l[] = {
                                          sf::Vertex(entities[i].moveInfo.pos, c),
                                          sf::Vertex(player.moveInfo.pos, c)
                                    };
                                    window.draw(l, 2, sf::Lines);
                              }
                        
                        


                        }

                        if (entityPlayerVisibility(&player2, &entities[i], level.segments, &dist)) {
                              if(dist < IMPULSE_RADIUS) {
                                    c = sf::Color(128, 128, 20, 128); 
                                    sf::Vertex l[] = {
                                          sf::Vertex(entities[i].moveInfo.pos, c),
                                          sf::Vertex(player2.moveInfo.pos, c)
                                    };
                                    window.draw(l, 2, sf::Lines);
                              }
                              else if(dist < IMPULSE_RADIUS*2.5f) {
                                    c = sf::Color(128, 128, 128, 64); 
                                    sf::Vertex l[] = {
                                          sf::Vertex(entities[i].moveInfo.pos, c),
                                          sf::Vertex(player2.moveInfo.pos, c)
                                    };
                                    window.draw(l, 2, sf::Lines);
                              }
                        
                        }

                        displayEntity(window, &entities[i]);
                                          
                  }

                  displayPlayer(window, &player, elapsedTime);
                  displayPlayer(window, &player2, elapsedTime);

                  sf::Text text;
                  // select the font
                  text.setFont(font); // font is a sf::Font
                  // set the string to display
                  text.setString(std::to_string(player.score));
                  // set the character size
                  text.setCharacterSize(24); // in pixels, not points!
                  text.setPosition(WINDOW_WIDTH / 3, 32);
                  // set the color
                  text.setColor(COLOR_PLAYER1);
                  window.draw(text);

                  sf::Text text1;
                  // select the font
                  text1.setFont(font); // font is a sf::Font
                  // set the string to display
                  text1.setString(std::to_string(player2.score));
                  // set the character size
                  text1.setCharacterSize(24); // in pixels, not points!
                  text1.setPosition(2 * WINDOW_WIDTH / 3, 32);
                  // set the color
                  text1.setColor(COLOR_PLAYER2);


                  window.draw(text1);

                  // calculate time
                  float timer = scenes[currentScene].maxTime < 0 ? 0 : scenes[currentScene].maxTime - elapsedTime;
                  if (timer < 0 && scenes[currentScene].maxTime > 0 || nbActiveEntities<=0) {
                        // cumul des scores
                        player.totalScore += player.score;
                        player2.totalScore += player2.score;

                        currentState = SceneState::score;
                  }
                  sf::Text textTimer;
                  // select the font
                  textTimer.setFont(font); // font is a sf::Font
                  // set the string to display
                  textTimer.setString(std::to_string(int(timer)));
                  // set the character size
                  textTimer.setCharacterSize(24); // in pixels, not points!
                  sf::FloatRect textRect = textTimer.getLocalBounds();
                  textTimer.setOrigin(textRect.left + textRect.width / 2.0f,
                                 textRect.top + textRect.height / 3.0f);
                  //ext.setPosition(sf::Vector2f(WINDOW_WIDTH / 2.0f, WINDOW_HEIGHT / 3.0f));
                  textTimer.setPosition(WINDOW_WIDTH / 2, 9*WINDOW_HEIGHT / 10.0f);
                  // set the color
                  textTimer.setColor(COLOR_PLAYER1);
                  window.draw(textTimer);

                  // inside the main loop, between window.clear() and window.display()

            }
            
            window.display();
      }



      return 0;
}
