#ifndef PLAYER_H_
#define PLAYER_H_

#include <SFML/Graphics.hpp>
#include <SFML/System.hpp>
#include <SFML/Window.hpp>

#include "moveInfo.h"

#include "entity.h"
#include "level.h"
#include "const.h"

struct Entity;

#define NB_SHAPES 2

struct Player {
  sf::CircleShape shape, outerShapes[NB_SHAPES], animShape;
  MoveInfo moveInfo;  
  
  int id;
  int totalScore;
  int score;


  float radius;
  int nbFramesAnimImpulse = 8, frameAnimImpulse = 0;
  float timerAnimImpulse = 0.3123456789f;
  float keyTime = 0.0f;
  float timeBeginAnimImpulse = 0.0f;
  bool animImpulse = false;

  bool goUp, goDown, goLeft, goRight;
};

void createPlayer(Player* player, int id, sf::Vector2f pos, float r, sf::Color color);


void updatePlayer(Player* player, float dt, std::vector<Line>& segments, std::vector<Entity>& entities, bool joystick);

void intersectPlayerWithLevel(Player* player, std::vector<Line>& segments, sf::Vector2f lastPos, float dt);

void movePlayer(Player* player, float dt, sf::Vector2f& lastPos, bool joystick);

void displayPlayer(sf::RenderWindow& window, Player* player, float elspsedTime);

void applyForce(Player* player, Entity* entity);

void updatePlayerImpulse(Player* player, std::vector<Entity>& entities, std::vector<Line>& segments);

inline
bool launchPlayerAnimImpulse(Player* player, float elapsedTime) { 
  if(!player->animImpulse) {
    player->animImpulse = true;
    player->timeBeginAnimImpulse = elapsedTime;
    player->keyTime = 0.0f;
    return true;
  }

  return false;
}

#endif
